@@ -21,7 +21,7 @@
 """Compatibility tests between the Dulwich client and the cgit server."""
 
 import copy
-from io import BytesIO
+import http.server
 import os
 import select
 import signal
@@ -31,38 +31,21 @@ import sys
 import tarfile
 import tempfile
 import threading
-
+from contextlib import suppress
+from io import BytesIO
 from urllib.parse import unquote
 
-import http.server
-
-from dulwich import (
-    client,
-    file,
-    index,
-    protocol,
-    objects,
-    repo,
-)
-from dulwich.tests import (
-    SkipTest,
-    expectedFailure,
-)
-from dulwich.tests.compat.utils import (
-    CompatTestCase,
-    check_for_daemon,
-    import_repo_to_dir,
-    rmtree_ro,
-    run_git_or_fail,
-    _DEFAULT_GIT,
-)
+from dulwich import client, file, index, objects, protocol, repo
+from dulwich.tests import SkipTest, expectedFailure
 
+from .utils import (_DEFAULT_GIT, CompatTestCase, check_for_daemon,
+                    import_repo_to_dir, rmtree_ro, run_git_or_fail)
 
 if sys.platform == "win32":
     import ctypes
 
 
-class DulwichClientTestBase(object):
+class DulwichClientTestBase:
     """Tests for client/server compatibility."""
 
     def setUp(self):
@@ -248,12 +231,10 @@ class DulwichClientTestBase(object):
                 dest.refs.set_if_equals(r[0], None, r[1])
             self.assertEqual(
                 dest.get_shallow(),
-                set(
-                    [
-                        b"35e0b59e187dd72a0af294aedffc213eaa4d03ff",
-                        b"514dc6d3fbfe77361bcaef320c4d21b72bc10be9",
-                    ]
-                ),
+                {
+                    b"35e0b59e187dd72a0af294aedffc213eaa4d03ff",
+                    b"514dc6d3fbfe77361bcaef320c4d21b72bc10be9",
+                },
             )
 
     def test_repeat(self):
@@ -331,7 +312,7 @@ class DulwichClientTestBase(object):
             sendrefs[b"refs/heads/abranch"] = b"00" * 20
             del sendrefs[b"HEAD"]
 
-            def gen_pack(have, want, ofs_delta=False):
+            def gen_pack(have, want, ofs_delta=False, progress=None):
                 return 0, []
 
             c = self._client()
@@ -346,7 +327,7 @@ class DulwichClientTestBase(object):
             dest.refs[b"refs/heads/abranch"] = dummy_commit
             sendrefs = {b"refs/heads/bbranch": dummy_commit}
 
-            def gen_pack(have, want, ofs_delta=False):
+            def gen_pack(have, want, ofs_delta=False, progress=None):
                 return 0, []
 
             c = self._client()
@@ -406,11 +387,9 @@ class DulwichTCPClientTest(CompatTestCase, DulwichClientTestBase):
             ctypes.windll.kernel32.TerminateProcess(handle, -1)
             ctypes.windll.kernel32.CloseHandle(handle)
         else:
-            try:
+            with suppress(OSError):
                 os.kill(pid, signal.SIGKILL)
                 os.unlink(self.pidfile)
-            except (OSError, IOError):
-                pass
         self.process.wait()
         self.process.stdout.close()
         self.process.stderr.close()
@@ -435,7 +414,7 @@ class DulwichTCPClientTest(CompatTestCase, DulwichClientTestBase):
         self.skipTest('skip flaky test; see #1015')
 
 
-class TestSSHVendor(object):
+class TestSSHVendor:
     @staticmethod
     def run_command(
         host,
@@ -648,7 +627,7 @@ class HTTPGitServer(http.server.HTTPServer):
         self.server_name = "localhost"
 
     def get_url(self):
-        return "http://%s:%s/" % (self.server_name, self.server_port)
+        return "http://{}:{}/".format(self.server_name, self.server_port)
 
 
 class DulwichHttpClientTest(CompatTestCase, DulwichClientTestBase):
